﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WebEdition;
using System.Configuration;

namespace Geosys.Prototypes.MapSuite.WebEdition.Pages.multipoint
{
    public partial class ModuleCarto : System.Web.UI.UserControl
    {



        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                InitializeModuleCarto();


            }
        }

        #region Module Carto Initialisation

        private void InitializeModuleCarto()
        {
            InitializeInchedPerUnit();
            InitializeMapAppearance();
            InitializeBackgroundLayer();
            InitializeMapExtend();
            InitializeFeature();
            Map1.MapUnit = GeographyUnit.Meter;

        }

        #region Init Current Scale
        public const int DotsPerInch = 96;
        public Dictionary<string, double> inchesPerUnits;

        public void InitializeInchedPerUnit()
        {
            inchesPerUnits = new Dictionary<string, double>();
            inchesPerUnits.Add("DecimalDegree", 4374754);
            inchesPerUnits.Add("Feet", 12.0);
            inchesPerUnits.Add("Meter", 39.3701);
        }
        #endregion

        private void InitializeMapAppearance()
        {
            Map1.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#B3C6D4"));
            Map1.MapTools.OverlaySwitcher.Enabled = false;
            Map1.MapTools.MouseCoordinate.Enabled = true;
            Map1.MapTools.PanZoomBar.Enabled = true;
            Map1.MapTools.ScaleLine.Enabled = true;
            Map1.MapTools.Logo.Enabled = true;
            Map1.EditOverlay.EditSettings.IsDraggable = true;
            Map1.EditOverlay.EditSettings.IsReshapable = true;
            Map1.EditOverlay.EditSettings.IsResizable = false;
            Map1.EditOverlay.EditSettings.IsRotatable = true;

            Map1.EditOverlay.Style.FillColor = GeoColor.SimpleColors.BrightYellow;
            Map1.EditOverlay.Style.OutlineColor = GeoColor.SimpleColors.Blue;
            Map1.EditOverlay.Style.OutlineWidth = 2;
        }

        private void InitializeBackgroundLayer()
        {
            GoogleOverlay gOverlay = new GoogleOverlay();
            gOverlay.Name = "Google Overlay";
            gOverlay.GoogleMapType = GoogleMapType.Hybrid;
            string sOverlayMapURL = ConfigurationManager.AppSettings["sOverlayMapURL"];
            gOverlay.JavaScriptLibraryUri = new Uri(sOverlayMapURL);
            gOverlay.IsBaseOverlay = true;
            Map1.CustomOverlays.Add(gOverlay);

        }

        private void InitializeMapExtend()
        {
        }


        private BaseShape ProjectGeometry(BaseShape shapeIn, string projIn, string projOut)
        {
            Proj4Projection proj4 = new Proj4Projection();

            proj4.InternalProjectionParametersString = projIn;
            proj4.ExternalProjectionParametersString = projOut;
            proj4.Open();

            BaseShape shapeOut = proj4.ConvertToExternalProjection(shapeIn);

            proj4.Close();
            return shapeOut;
        }



        #endregion



        private void InitializeFeature()
        {
            initialiseField();
            initialisePoints();
        }


        private void initialiseField()
        {
            //string wktField = "MULTIPOLYGON(((1.44470214823636 43.6512302483965,1.40590667705037 43.5759139818001,1.55834197976355 43.5724317425177,1.53671264627045 43.6445227261031,1.44470214823636 43.6512302483965)))";
            //MultipolygonShape multi = MultipolygonShape.CreateShapeFromWellKnownData(wktField) as MultipolygonShape;

            //BaseShape bs = BaseShape.CreateShapeFromWellKnownData(wktField);
            //BaseShape bs2 = ProjectGeometry(bs, Proj4Projection.GetEpsgParametersString(4326), Proj4Projection.GetGoogleMapParametersString());
            //InMemoryFeatureLayer imLayer = new InMemoryFeatureLayer();
            //imLayer.Open();
            //imLayer.FeatureSource.BeginTransaction();
            //imLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush = new GeoSolidBrush(GeoColor.StandardColors.White);
            //imLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen = new GeoPen(GeoColor.StandardColors.Black);
            //imLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //Feature feature = bs2.GetFeature();
            //imLayer.FeatureSource.AddFeature(feature);
            //imLayer.FeatureSource.CommitTransaction();
            //imLayer.FeatureSource.Close();

            //LayerOverlay lo = new LayerOverlay();
            //lo.IsBaseOverlay = false;
            //lo.Layers.Add(imLayer);

            //Map1.CustomOverlays.Add(lo);

            //Map1.CurrentExtent = bs2.GetBoundingBox();

        }

        private void initialisePoints()
        {
            ////multipoint
            //MultipointShape multipoint = MultipointShape.CreateShapeFromWellKnownData("MULTIPOINT(-68487.5773375 6085610.4431062, 293518.18855625 6085610.4431062, 293518.18855625 5655117.09988121)") as MultipointShape;
            //// On vide la couche d’édition
            //if (!Map1.EditOverlay.FeatureSource.IsOpen) Map1.EditOverlay.FeatureSource.Open();
            //Map1.EditOverlay.FeatureSource.InternalFeatures.Clear();
            //Feature feature = new Feature(multipoint.GetWellKnownBinary(), "MultiPoint");

            //Map1.EditOverlay.FeatureSource.BeginTransaction();
            //// On ajoute le multipolygone à la couche d’édition
            //Map1.EditOverlay.FeatureSource.AddFeature(feature);
            //Map1.EditOverlay.FeatureSource.CommitTransaction();
            //if (Map1.EditOverlay.FeatureSource.IsOpen) Map1.EditOverlay.FeatureSource.Close();
        }

        protected void btnCreateMultiPoints_Clicked(object sender, EventArgs eas)
        {
            //todo 
            //limiter le nombre de points 
            //attention si la distance entre les points > longueur de la bbox on aura pas de point dans la parcelle

            //double nbKmBetweenPoints = Convert.ToDouble(txtDistance1.Text);
            double nbKmBetweenPoints = 5;
            

            List<string> listWKT = new List<string>();

            string wktField = "MULTIPOLYGON(((1.44470214823636 43.6512302483965,1.40590667705037 43.5759139818001,1.55834197976355 43.5724317425177,1.53671264627045 43.6445227261031,1.44470214823636 43.6512302483965)))";
            BaseShape bs = BaseShape.CreateShapeFromWellKnownData(wktField);
            BaseShape bs2 = ProjectGeometry(bs, Proj4Projection.GetEpsgParametersString(4326), Proj4Projection.GetGoogleMapParametersString());
            MultipolygonShape multi = MultipolygonShape.CreateShapeFromWellKnownData(bs2.GetWellKnownText()) as MultipolygonShape;
            
            RectangleShape bbox = multi.GetBoundingBox();
            PointShape llp = bbox.LowerLeftPoint;

            bool isInsideHorizontal = true;
            bool isInsideVertical = true;


            PointShape pointCurrentVertical = llp.CloneDeep() as PointShape;
            bool abort = false;
            while (isInsideVertical && !abort)
            {
                PointShape pointCurrentHorizontal = pointCurrentVertical.CloneDeep() as PointShape;
                isInsideHorizontal = true;
                while (isInsideHorizontal && !abort)
                {
                    pointCurrentHorizontal.TranslateByOffset(nbKmBetweenPoints, 0, GeographyUnit.Meter, DistanceUnit.Kilometer);
                    if (bbox.Contains(pointCurrentHorizontal))
                    {
                        if (multi.Contains(pointCurrentHorizontal))
                            listWKT.Add(pointCurrentHorizontal.GetWellKnownText());
                    }
                    else
                        isInsideHorizontal = false;

                    if (listWKT.Count > 200) //par mesure de securite et pour les performances
                        abort = true;
                }

                pointCurrentVertical.TranslateByOffset(0, nbKmBetweenPoints, GeographyUnit.Meter, DistanceUnit.Kilometer);
                if (bbox.Contains(pointCurrentVertical))
                {
                    if (multi.Contains(pointCurrentVertical))
                        listWKT.Add(pointCurrentVertical.GetWellKnownText());
                }
                else
                    isInsideVertical = false;

            }



            MultipointShape multiPoint = new MultipointShape();
            foreach (string wkt in listWKT)
            {
                PointShape point = PointShape.CreateShapeFromWellKnownData(wkt) as PointShape;
                multiPoint.Points.Add(point);
            }


            InMemoryFeatureLayer imLayer = new InMemoryFeatureLayer();
            imLayer.Open();
            imLayer.FeatureSource.BeginTransaction();
            imLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush = new GeoSolidBrush(GeoColor.StandardColors.White);
            imLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen = new GeoPen(GeoColor.StandardColors.Black);
            imLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            imLayer.FeatureSource.AddFeature(multi.GetFeature());
            imLayer.FeatureSource.CommitTransaction();
            imLayer.FeatureSource.Close();

            LayerOverlay lo = new LayerOverlay();
            lo.IsBaseOverlay = false;
            lo.Layers.Add(imLayer);

            Map1.CustomOverlays.Add(lo);

            Map1.CurrentExtent = bs2.GetBoundingBox();

            // On vide la couche d’édition
            if (!Map1.EditOverlay.FeatureSource.IsOpen) Map1.EditOverlay.FeatureSource.Open();
            Map1.EditOverlay.FeatureSource.InternalFeatures.Clear();
            Feature feature = new Feature(multiPoint.GetWellKnownBinary(), "MultiPoint");

            Map1.EditOverlay.FeatureSource.BeginTransaction();
            // On ajoute le multipolygone à la couche d’édition
            Map1.EditOverlay.FeatureSource.AddFeature(feature);
            Map1.EditOverlay.FeatureSource.CommitTransaction();
            if (Map1.EditOverlay.FeatureSource.IsOpen) Map1.EditOverlay.FeatureSource.Close();


           
        }

    }
}